/*
 * Vertices in strip are numbered as follows:
 *
 *     ___ X-2 ___ X-1 ___ 0   ___ 1   ___ 2   ___ 3   ___ 4   __
 *         Y-1     Y-1      0       0       0       0       0
 *        /  \    /  \    /  \    /  \    /  \    /  \    /  \
 *       /    \  /    \  /    \  /    \  /    \  /    \  /    \
 *     X-3 ___ X-2 ___ X-1 ___ 0   ___ 1   ___ 2   ___  3  ___ 4
 *     Y-2     Y-2     Y-2      1       1       1       1       1
 *       \    /  \    /  \    /  \    /  \    /  \    /  \    /
 *        \  /    \  /    \  /    \  /    \  /    \  /    \  /
 *     ___ X-3 ___ X-2 ___ X-1 ___ 0   ___ 1   ___ 2   ___ 3   __
 *         Y-3     Y-3     Y-3      2       2       2       2
 *
 *       .   .   .   .   .   .   .   .   .   .   .   .   .   .   .
 *
 *      X-4 ___ X-3 ___ X-2 ___ X-1 ___ 0   ___ 1   ___ 2   ___ 3
 *       1       1       1       1      Y-2     Y-2     Y-2     Y-2
 *        \    /  \    /  \    /  \    /  \    /  \    /  \    /
 *         \  /    \  /    \  /    \  /    \  /    \  /    \  /
 *      ___ X-4 ___ X-3 ___ X-2 ___ X-1 ___ 0   ___ 1   ___ 2   __
 *           0       0       0       0      Y-1     Y-1     Y-1
 *
 * Node x,y for
 *   0 <= x < X     x = distance along
 *   0 <= y < Y     y = distance across
 *
 * Vertices are in reading order from diagram above ie x varies fastest.
 *
 * Y must be even.  The actual location opposite (0,0) is (X-(Y-1)/2,0),
 * and likewise opposite (0,Y-1) is ((Y-1)/2,0).
 *
 * We label edges as follows:
 *
 *			    \0   /1
 *			     \  /
 *			  ___ 0   __
 *			  2    1   3
 *			     /  \
 *			   4/   5\
 *
 * (This numbering permits the order-4 nodes at the strip's edge
 *  to have a contiguous edge numbering 2..5 or 0..3.)
 *
 * When we iterate over edges, we iterate first over vertices and then
 * over edges 0 to 2, disregarding edges 3 to 5.
 */

#ifndef MGRAPH_H
#define MGRAPH_H

#define XBITS 4
#define X (1<<XBITS)
#define YBITS 4
#define Y (1<<YBITS)

/* vertex number:   0000 | y     | x
 *                        YBITS   XBITS
 */

#define N (X*Y)
#define XMASK (X-1)
#define YSHIFT XBITS
#define Y1 (1 << YSHIFT)
#define YMASK (Y-1 << YSHIFT)

#define V6 6

#define D3 3

#define FOR_VERTEX(v) \
  for ((v)=0; (v)<N; (v)++)

#define VE_MIN(v) ((v) & YMASK ? 0 : 2)
#define VE_MAX(v) (~(v) & YMASK ? V6 : 4)

#define FOR_VPEDGE(v,e) \
  for ((e)=VE_MIN(v); (e)<VE_MAX(v); (e)++)

extern int edge_end2(unsigned v1, int e);
#define EDGE_END2 edge_end2

#define FOR_VEDGE(v1,e) \
  FOR_VPEDGE((v1),(e))
    if (((v2)= EDGE_END2((v1),(e))) < 0) ; else

#define FOR_EDGE(v1,e,v2) \
  FOR_VERTEX((v1)) \
    FOR_VEDGE((v1),(e),(v2))

#define FOR_COORD(k) \
  for ((k)=0; (k)<D3; (k)++)

#define K FOR_COORD(k)

typedef struct {
  double v[N][D3];
} Layout;

#endif /*MGRAPH_H*/
